==> spellathon-5.59.tar.gz: spellathon-5.59/src/gui/wordinput.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QLineEdit>
#include <QPainter>
#include <QKeyEvent>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QGridLayout>
#include <QLinearGradient>

#include "wordinput.h"

WordInput::WordInput(QWidget *p) 
	: QWidget(p)
{ 
	wordEdit = new WordEdit(this);
	chkImg = new QLabel(this);
	chkText = new QLabel(this);

	QSizePolicy szpol1(QSizePolicy::Expanding, 
					   QSizePolicy::Fixed, QSizePolicy::LineEdit);
	szpol1.setHorizontalStretch(0);
	szpol1.setVerticalStretch(0);
	wordEdit->setSizePolicy(szpol1);

	QSizePolicy szpol2(QSizePolicy::Preferred, QSizePolicy::Preferred,
					   QSizePolicy::Label);
	szpol2.setHorizontalStretch(0);
	szpol2.setVerticalStretch(0);
	chkImg->setSizePolicy(szpol2);

	QSizePolicy szpol3(QSizePolicy::Expanding, QSizePolicy::Preferred,
					   QSizePolicy::Label);
	szpol3.setHorizontalStretch(0);
	szpol3.setVerticalStretch(0);
	chkText->setSizePolicy(szpol3);
	chkText->setMargin(0);
	chkText->setIndent(5);

	hzspcer = new QSpacerItem(0, 0, QSizePolicy::Expanding, 
							  QSizePolicy::Preferred);

	grdlayout = new QGridLayout(this);
    grdlayout->setMargin(2);
	grdlayout->addWidget(wordEdit, 0, 0);
	grdlayout->addWidget(chkImg, 0, 1);
	grdlayout->addWidget(chkText, 0, 2);
	grdlayout->addItem(hzspcer, 0, 3);

	setContextMenuPolicy(Qt::NoContextMenu); 
	setFocusProxy(wordEdit);

	setIndicator(BLANK);

	connect(wordEdit, SIGNAL(setIndicator(int)), this, SLOT(setIndicator(int)));
}

void
WordInput::paintEvent(QPaintEvent *ev)
{
        QPainter p(this);
        p.setPen(Qt::NoPen);
		QRect crect = contentsRect();

		QLinearGradient lgrad(0, 0, crect.width(), crect.height());
		if (chkState == BLANK) {
			lgrad.setColorAt(0, palette().color(QPalette::Window).darker(106));
			lgrad.setColorAt(1, palette().color(QPalette::Window));
		} else {
			QColor gc;
			switch (chkState) {
			case CORRECT:
				gc = 0xffb0e9b1;
				break;
			case WRONG:
				gc = 0xfff78585;
				break;
			case REPEAT:
				gc = 0xff92cbd3;
				break;
			}
			lgrad.setColorAt(0, gc);
			lgrad.setColorAt(1, Qt::white);
		}
		p.setBrush(QBrush(lgrad));

		p.drawRect(crect);
		ev->accept();
}

void
WordInput::setIndicator(int v)
{
	switch (chkState = v) {
	case WRONG:
		chkImg->setPixmap(QPixmap(QString(":/cross.png")));
		chkText->setText("<i><font color=\"slateblue\">Wrong!</font></i>");
		break;
	case CORRECT:
		chkImg->setPixmap(QPixmap(QString(":/tick.png")));
		chkText->setText("<i><font color=\"slateblue\">Correct!</font></i>");
		break;
	case REPEAT:
		chkImg->setPixmap(QPixmap(QString(":/repeat.png")));
		chkText->setText("<i><font color=\"slateblue\">Repeated</font></i>");
		break;
	case BLANK:
		chkImg->clear();
		chkText->clear();
		break;
	}

	update();
}

QSize 
WordInput::sizeHint() const 
{ 
	return QSize(wordEdit->sizeHint().width()
				 + chkText->sizeHint().width()
				 + chkImg->sizeHint().width()
				 , wordEdit->sizeHint().height() + 4); 
}

WordEdit::WordEdit(QWidget *p) 
	: QLineEdit(p), init_state(true), timer(0)
{ 
	setContextMenuPolicy(Qt::NoContextMenu); 
	timer = new QTimer(this);
	timer->setSingleShot(true);
	timer->setInterval(2000);
	connect(timer, SIGNAL(timeout()), this, SLOT(tmout_clear()));	
}

void 
WordEdit::setInitState(bool s) 
{ 
	init_state = s; 
	timer->stop();
	if (init_state) {
		setStyleSheet("color: grey; font: italic;");
		setText("Enter a word");
		setCursorPosition(0);
	} else {
		setStyleSheet("color: black; selection-background-color: #bad8ff; selection-color: black;");
		clear();
	}
	emit setIndicator(WordInput::BLANK);
}

void 
WordEdit::mousePressEvent(QMouseEvent *ev)
{
	if (init_state) 
		setInitState(false);

	emit setIndicator(WordInput::BLANK);

	QLineEdit::mousePressEvent(ev);
	ev->accept();
}

void 
WordEdit::keyPressEvent(QKeyEvent *ev)
{
	int k = ev->key();
	if (init_state) {
		if (k >= Qt::Key_A && k <= Qt::Key_Z) {
			setInitState(false);
			QLineEdit::keyPressEvent(ev);
		}
	} else {
		if (
			 (k >= Qt::Key_A && k <= Qt::Key_Z) || 
			 k == Qt::Key_Return || k == Qt::Key_Enter || 
			 k == Qt::Key_Left || k == Qt::Key_Right || 
			 k == Qt::Key_Home || k == Qt::Key_End || 
			 k == Qt::Key_Delete || k == Qt::Key_Backspace
		   ) 
		{
			timer->stop();
			emit setIndicator(WordInput::BLANK);
			if ((k != Qt::Key_Return && k != Qt::Key_Enter) || text() != "")
				QLineEdit::keyPressEvent(ev);
		}
	}
	ev->accept();
}

void 
WordEdit::focusInEvent(QFocusEvent *ev)
{
	QLineEdit::focusInEvent(ev);
	if (init_state)
		setCursorPosition(0);
}

void 
WordEdit::chk_clear() 
{ 
	clear();
	emit setIndicator(WordInput::BLANK);
}

void
WordEdit::tmout_clear()
{
	emit setIndicator(WordInput::BLANK);
}

void 
WordEdit::chk(int v)
{
	setCursorPosition(0);
	if (v == WordInput::CORRECT) {
		clear();
	} else {
		selectAll();
	}
	timer->start();
	emit setIndicator(v);
}

==> spellathon-5.72.tar.gz: spellathon-5.72/src/gui/wordinput.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QLineEdit>
#include <QPainter>
#include <QKeyEvent>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QGridLayout>
#include <QLinearGradient>

#include "wordinput.h"

WordInput::WordInput(QWidget *p) 
	: QWidget(p)
{ 
	wordEdit = new WordEdit(this);
	chkImg = new QLabel(this);
	chkText = new ChkText(this);

	QSizePolicy szpol1(QSizePolicy::Expanding, 
					   QSizePolicy::Preferred, QSizePolicy::LineEdit);
	szpol1.setHorizontalStretch(0);
	szpol1.setVerticalStretch(0);
	wordEdit->setSizePolicy(szpol1);

	QSizePolicy szpol2(QSizePolicy::Minimum, QSizePolicy::Preferred,
					   QSizePolicy::Label);
	szpol2.setHorizontalStretch(0);
	szpol2.setVerticalStretch(0);
	chkImg->setSizePolicy(szpol2);
	chkImg->setMinimumWidth(16);

	QSizePolicy szpol3(QSizePolicy::Preferred, QSizePolicy::Preferred,
					   QSizePolicy::Label);
	szpol3.setHorizontalStretch(0);
	szpol3.setVerticalStretch(0);
	chkText->setSizePolicy(szpol3);
	chkText->setIndent(4);

	setContentsMargins(0, 0, 0, 0);

	grdlayout = new QGridLayout(this);
	grdlayout->setContentsMargins(8, 4, 4, 4);
	grdlayout->addWidget(wordEdit, 0, 0);
	grdlayout->addWidget(chkImg, 0, 1);
	grdlayout->addWidget(chkText, 0, 2);

	setContextMenuPolicy(Qt::NoContextMenu); 
	setFocusProxy(wordEdit);

	setIndicator(BLANK);

	connect(wordEdit, SIGNAL(setIndicator(int)), this, SLOT(setIndicator(int)));
}

void
WordInput::paintEvent(QPaintEvent *ev)
{
        QPainter p(this);
		QRect crect = contentsRect().adjusted(0, 0, -1, -1);

		QLinearGradient lgrad(crect.width()/2, crect.height()/2, crect.width()/2, 0);
		lgrad.setSpread(QGradient::ReflectSpread);
		if (indicatorState == BLANK) {
			lgrad.setColorAt(0, palette().color(QPalette::Window).darker(110));
			lgrad.setColorAt(1, 0xf8f8f8);
		} else {
			QColor gc;
			switch (indicatorState) {
			case CORRECT:
				gc = 0xff8ada8a;
				break;
			case WRONG:
				gc = 0xfff78585;
				break;
			case REPEAT:
				gc = 0xff92cbd3;
				break;
			}
			lgrad.setColorAt(0, gc);
			lgrad.setColorAt(1, 0xf8f8f8);
		}
		p.setPen(QColor(0xc8c8c8));
		p.setBrush(QBrush(lgrad));

		p.drawRect(crect);
		ev->accept();
}

void
WordInput::setIndicator(int v)
{
	switch (indicatorState = v) {
	case WRONG:
		chkImg->setPixmap(QPixmap(QString(":/cross.png")));
		chkText->setText("<i><font size=\"+1\" color=\"slateblue\">Wrong!</font></i>");
		break;
	case CORRECT:
		chkImg->setPixmap(QPixmap(QString(":/tick.png")));
		chkText->setText("<i><font size=\"+1\" color=\"slateblue\">Correct!</font></i>");
		break;
	case REPEAT:
		chkImg->setPixmap(QPixmap(QString(":/repeat.png")));
		chkText->setText("<i><font size=\"+1\" color=\"slateblue\">Repeated</font></i>");
		break;
	case BLANK:
		chkImg->clear();
		chkText->clear();
		break;
	}

	update();
}

QSize 
WordInput::sizeHint() const 
{ 
	return QSize(wordEdit->sizeHint().width() 
				 + chkText->sizeHint().width()
				 + chkImg->sizeHint().width() + 6*4
				 , wordEdit->sizeHint().height() + 4); 
}

QSize
WordEdit::sizeHint() const
{
	QFont f = font();
	f.setItalic(true);
	QFontMetrics fm(f);
	int width = fm.width(QString("Enter a word ")) + 32;
	return QSize(width < 100 ? 100 : width, fm.height());
}

QSize 
ChkText::sizeHint() const 
{ 
	QFont f = font();
	f.setPointSize(f.pointSize() + 1);
	QFontMetrics fm(f);
	return QSize(fm.width(QString("Repeated ")) + 32, 
				 fm.height()); 
}

WordEdit::WordEdit(QWidget *p) 
	: QLineEdit(p), init_state(true), timer(0)
{ 
	setContextMenuPolicy(Qt::NoContextMenu); 
	timer = new QTimer(this);
	timer->setSingleShot(true);
	timer->setInterval(2000);
	connect(timer, SIGNAL(timeout()), this, SLOT(tmout_clear()));	
}

void 
WordEdit::setInitState(bool s) 
{ 
	init_state = s; 
	timer->stop();
	if (init_state) {
		setStyleSheet("color: grey; font: italic;");
		setText("Enter a word");
		setCursorPosition(0);
	} else {
		setStyleSheet("color: normal; selection-background-color: #bad8ff; selection-color: black;");
		clear();
	}
	emit setIndicator(WordInput::BLANK);
}

void 
WordEdit::mousePressEvent(QMouseEvent *ev)
{
	if (init_state) 
		setInitState(false);

	emit setIndicator(WordInput::BLANK);

	if (chk_state == WordInput::CORRECT)
		clear();
	else
		QLineEdit::mousePressEvent(ev);
	ev->accept();
}

void 
WordEdit::keyPressEvent(QKeyEvent *ev)
{
	int k = ev->key();
	if (init_state) {
		if (k >= Qt::Key_A && k <= Qt::Key_Z) {
			setInitState(false);
			QLineEdit::keyPressEvent(ev);
		}
	} else {
		if (
			 (k >= Qt::Key_A && k <= Qt::Key_Z) || 
			 k == Qt::Key_Return || k == Qt::Key_Enter || 
			 k == Qt::Key_Left || k == Qt::Key_Right || 
			 k == Qt::Key_Home || k == Qt::Key_End || 
			 k == Qt::Key_Delete || k == Qt::Key_Backspace
		   ) 
		{
			timer->stop();
			emit setIndicator(WordInput::BLANK);
			if ((k != Qt::Key_Return && k != Qt::Key_Enter) || text() != "")
				QLineEdit::keyPressEvent(ev);
		}
	}
	ev->accept();
}

void 
WordEdit::focusInEvent(QFocusEvent *ev)
{
	QLineEdit::focusInEvent(ev);
	if (init_state)
		setCursorPosition(0);
}

void 
WordEdit::chk_clear() 
{ 
	clear();
	emit setIndicator(WordInput::BLANK);
}

void
WordEdit::tmout_clear()
{
	emit setIndicator(WordInput::BLANK);
	if (chk_state == WordInput::CORRECT)
		clear();

}

void 
WordEdit::chk(int v)
{
	chk_state = v;
	setCursorPosition(0);
	selectAll();
	timer->start();
	emit setIndicator(v);
}
